////////////////////////////////////////////////////////////////////////////////
///                            MeshDrawer.h                                  ///
////////////////////////////////////////////////////////////////////////////////

#ifndef MESH_DRAWER_H
#define MESH_DRAWER_H

#include "MeshDrawerGlobalVariables.h"

#include <X3DTK/kernel.h>
#include <X3DTK/meshscenegraph.h>

namespace X3DTK {

enum SelectedType {NOTHING, VERTICES, EDGES, FACES};

namespace MESH {

// Processor drawing the mesh from the Mesh scene graph.

class MeshDrawer : public X3DOnePassProcessor
{
public:
  MeshDrawer();
  virtual ~MeshDrawer();
  
  //  Setting parameters.
  void changeDrawPoints();
  void changeDrawEdges();
  void changeDrawFaces(); 
  inline void changeOneColorPerMesh() {globalVariables->changeOneColorPerMesh();}; 
  void setSelectedType(SelectedType selectedType);
  void setSelectedMesh(int selectedMesh);
  void setSelectedId(int selectedId);
  
  // Getting parameters.
  inline SelectedType getSelectedType() const {return selectedType;};
  inline int getSelectedMesh() const {return selectedMesh;};
  inline int getSelectedId() const {return selectedId;};
  
  // Draw methods.
  void drawMesh(X3DNode *N, bool selection = false);
  void drawSelected();
 
protected:
  MeshDrawerGlobalVariables* globalVariables;
  
  bool drawPoints, drawEdges, drawFaces;
  SelectedType selectedType;
  int selectedMesh;
  int selectedId;

  // Draw methods.
  void drawVertex(const Mesh::Vertex *vertex);
  void drawEdge(Mesh::Edge *edge);
  void drawFace(const Mesh::Face *face);
};

}
}

#endif

