#include "MeshDrawerGlobalVariables.h"
#include <qglobal.h>

namespace X3DTK {
namespace MESH {

MeshDrawerGlobalVariables::MeshDrawerGlobalVariables()
: GlobalVariables(), _selectionMode(false), _oneColorPerMesh(true), _primitiveType(1)
{}

void MeshDrawerGlobalVariables::initTraversal()
{
  _matrixStack.push_front(SFMatrix34f::IDENTITY);
}

void MeshDrawerGlobalVariables::finishTraversal()
{
  _matrixStack.pop_front();
  if (_matrixStack.size() != 0)
    {
      qWarning("Non empty matrix stack at the end of MeshDrawer traversal");
      _matrixStack.clear();
    }
  
  _matrixVector.clear();
  _meshVector.clear();
}

void MeshDrawerGlobalVariables::pushMatrix(const SFMatrix34f &transformation)
{
  _matrixStack.push_front(_matrixStack.front()*transformation);
}

void MeshDrawerGlobalVariables::popMatrix()
{
  _matrixStack.pop_front();
}

void MeshDrawerGlobalVariables::storeMesh(Mesh* mesh)
{
  _matrixVector.push_back(_matrixStack.front());
  _meshVector.push_back(mesh);
}

int MeshDrawerGlobalVariables::getCurrentMeshId()
{
  return _matrixVector.size() - 1;
}

}
}
