#include "Viewer.h"
#include <X3DTK/glrenderer.h>

#include <math.h>
#include <iostream>
#include <qfiledialog.h>

using namespace X3DTK;
using namespace std;

Viewer::Viewer(const char *file)
: normLength(1.0f), normal(false)
{
  x3dfile = (char *)file;
}

Viewer::~Viewer()
{
  scene.release();
}

void Viewer::keyPressEvent(QKeyEvent *e)
{
  switch (e->key())
  {
    case Qt::Key_L : 
      loadFile(); break;
    case Qt::Key_N : 
      normal = !normal;
      scene.setNormal(normal);
      break;
    case Qt::Key_Minus :
      normLength /= 2.0f;
      scene.setNormalLength(normLength);
      break;
    case Qt::Key_Plus :
      normLength *= 2.0f;
      scene.setNormalLength(normLength);
      break;
    default: 
      QGLViewer::keyPressEvent(e);
  }
  updateGL();
}

void Viewer::loadFile()
{
  QString name = QFileDialog::getOpenFileName("", "X3D files (*.x3d *.X3D);;All files (*)", this);
  
  // In case of Cancel
  if (name.isEmpty())
    return;

  // Loads the file name.
  scene.load(name, false);

  setSceneBoundingBox(scene.getBboxMin().f_data(), scene.getBboxMax().f_data());
  showEntireScene();
}

void Viewer::init()
{
  loadFile();
  glEnable(GL_RESCALE_NORMAL);
}

void Viewer::draw()
{
  //traverse the scene graph
  scene.draw();
}

