////////////////////////////////////////////////////////////////////////////////
//                            Info.h                                          //
////////////////////////////////////////////////////////////////////////////////

#ifndef INFO_H
#define INFO_H

#include <X3DTK/kernel.h>

namespace X3DTK {
namespace X3D {

class Price;
class Modeler;

class Info : public X3DChildNode
{
public:
  // Constructor.
  Info();
  // Constructs a Cylinder from its attributes.
  Info(const SFString &date,
       SFNode price,
       SFNode modeler);
            
  // Clones the node.
  virtual SFNode clone() const;  
  // sets the date.
  void setDate(const SFString &date);
  // sets the Price node.
  void setPrice(Price *price);
  // sets the Modeler node.
  void setModeler(Modeler *modeler);  

  // gets the date.
  inline SFString getDate() const {return _date;};
  // sets the Price node.
  inline SFNode getPrice() const {return _price;};
  // sets the Modeler node.
  inline SFNode getModeler() const {return _modeler;};  
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFAbstractNode &N);
  
  // Loads the attributes from a X3DFileElement e.
  virtual void load(const X3DFileElement *element);
  // Writes the attributes of the node.
  virtual SFString write() const;

protected:
  // Copy constructor.	   
  Info(const Info &I);

private:
  SFString _date;
  
  SFNode _price;
  SFNode _modeler;
};

}
}

#endif
