#include "Viewer.h"
#include "MyGeometry3DCreator.h"
#include "MyGLBuilderGeometry3DVisitor.h"

#include <X3DTK/kernel.h>

#include <qfiledialog.h>

using namespace X3DTK;
using namespace std;

void Viewer::init()
{
  // We customize the SimpleX3DGLScene, by changing its X3DLoader, GLCreator
  scene.getLoader()->setComponentCreator(new X3D::MyGeometry3DCreator());
  scene.getGLBuilder()->setComponentVisitor(new X3D::MyGLBuilderGeometry3DVisitor());
  
  glEnable(GL_RESCALE_NORMAL);
  help();
  loadFile();
}

void Viewer::keyPressEvent(QKeyEvent *e)
{
  switch (e->key())
  {
    case Qt::Key_L : loadFile(); break;
    default:         QGLViewer::keyPressEvent(e);
  }
}

void Viewer::loadFile()
{
  QString name = QFileDialog::getOpenFileName("", "X3D files (*.x3d *.X3D);;All files (*)", this);
  
  // In case of Cancel
  if (name.isEmpty())
    return;

  scene.release();
  // Loads the scene.
  scene.load(name);
  
  // Shows the derivation tree.
  Type::printDerivationTree(); 
  cout << endl;
  
  // QGLViewer settings.
  setSceneBoundingBox(scene.getBboxMin().f_data(), scene.getBboxMax().f_data());
  showEntireScene();
}

void Viewer::draw()
{
  // Draws the scene.
  scene.draw();
}

void Viewer::help()
{
  cout << endl << "\t\t- -  x 3 d V i e w e r  - -" << endl << endl;
  cout << "This example uses the libX3D library to load a x3d object file." << endl;
  cout << "Press 'L'(oad) to open a x3d file." << endl << endl;
}
