/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            Creator.h                                       //
////////////////////////////////////////////////////////////////////////////////

#ifndef CREATOR_H
#define CREATOR_H

#include "CreationFunction.h"
#include "X3DActor.h"

namespace X3DTK {
namespace X3D {

class X3DComponentCreator;

/// Concrete class defining a Creator, creating X3D nodes from their names.

class Creator : public X3DActor
{
public:  
  /// Constructor.
  Creator();
  /// Destructor.
  virtual ~Creator();
  
  /// Sets the X3DComponentCreator.
  void setComponentCreator(X3DComponentCreator *component);
  /// Creates the SFNode from its name.
  X3D::SFNode createFromName(const SFString &name) const;
  /// Empties the list of component.
  void reset();
  
  /// Gets the name of the component of the type name.
  SFString getComponentNameOf(const SFString &name) const;

  friend Creator *joinCreator(Creator *N0, Creator *N1);
  
private:  
  /// Adds a type.
  virtual void addType(const Type *type);
  ///Gets the creation function related with name.
  CreationFunction *getCreationFunctionOf(const SFString &name) const;
  
  //Dictionary of all the creation functions used.
  CreationDict creationDict_;
  std::list<X3DComponentCreator *> componentList_;
};

}
}

#endif
