/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GLBuilderGlobalVariables.h                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLBUILDERGLOBALVARIABLES_H
#define GLBUILDERGLOBALVARIABLES_H

#include "GlobalVariables.h"
#include "GL_X3DNode.h"

#include <map>

namespace X3DTK {
namespace X3D {

/// GlobalVariables for the GLBuilder module.

class GLBuilderGlobalVariables : public GlobalVariables
{
public:
  /// Constructor.
  GLBuilderGlobalVariables();
  /// Destructor.
  virtual ~GLBuilderGlobalVariables();
  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  virtual void finish();
  
  /// Adds the couple of nodes.
  void addCoupleNode(SFNode N, GL::SFNode GN);
  /// Pushes the node.
  void pushNode(GL::SFNode N);
  /// Pops the node.
  void popNode();
    
  /// Gets the GLScene.
  inline GL::SFNode getRoot() const {return _root;};
  /// Gets the top of the stack.
  inline GL::SFNode getTop() const {return (!_nodeStack.empty() ? _nodeStack.front() : 0);};
  /// Gets the GL::SFNode related with N. Returns 0 if N not found.
  GL::SFNode getNode(SFNode N) const;
  
private:
  GL::SFNode _root;
  GL::MFNode _nodeStack;
  std::map<SFNode, GL::SFNode> _nodeCoupleMap;
};

}
}

#endif
