/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            NURBSCreator.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef NURBSCREATOR_H
#define NURBSCREATOR_H

#include "X3DComponentCreator.h"

namespace X3DTK {
namespace X3D {

class Contour2D;
class ContourPolyline2D;
class NurbsCurve;
class NurbsCurve2D;
class NurbsGroup;
class NurbsPositionInterpolator;
class NurbsSurface;
class NurbsTextureSurface;
class TrimmedSurface;

/// Concrete component Creator for the NURBS component defining the  
/// X3D nodes.

class NURBSCreator : public X3DComponentCreator
{
public:
  /// Constructor.
  NURBSCreator();
  /// Destructor.
  virtual ~NURBSCreator();
  
  /// Creates a Contour2D.
  Contour2D *createContour2D() const;
  /// Creates a ContourPolyline2D.
  ContourPolyline2D *createContourPolyline2D() const;
  /// Creates a .
  NurbsCurve *createNurbsCurve() const;
  /// Creates a NurbsCurve2D.
  NurbsCurve2D *createNurbsCurve2D() const;
  /// Creates a NurbsGroup.
  NurbsGroup *createNurbsGroup() const;
  /// Creates a NurbsPositionInterpolator.
  NurbsPositionInterpolator *createNurbsPositionInterpolator() const;
  /// Creates a NurbsSurface.
  NurbsSurface *createNurbsSurface() const;
  /// Creates a NurbsTextureSurface.
  NurbsTextureSurface *createNurbsTextureSurface() const;
  /// Creates a TrimmedSurface.
  TrimmedSurface *createTrimmedSurface() const;
};

}
}

#endif
