/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            PositionInterpolator.h                          //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef POSITIONINTERPOLATOR
#define POSITIONINTERPOLATOR

#include "X3DTypes.h"
#include "X3DInterpolatorNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/interpolator.html">
/// Interpolator component</a> defining an PositionInterpolator
/// (<a href="../../doc/X3DSpecs/Part01/components/interpolator.html#PositionInterpolator">
/// documentation</a>).

class PositionInterpolator : public X3DInterpolatorNode
{
public:
  /// Constructor.
  PositionInterpolator();
  /// Constructs a PositionInterpolator from its attributes.
  PositionInterpolator(const MFFloat &key,
                       const MFVec3f &keyValue);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~PositionInterpolator();

  /// Sets the keyValue attribute.
  void setKeyValue(const MFVec3f &keyValue);

  /// Gets the keyValue attribute.
  inline const MFVec3f &getKeyValue() const {return _keyValue;};

  /// Loads the attributes from a X3DFileElement.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.
  PositionInterpolator(const PositionInterpolator &N);

private:
  /// Values of the piecewise-linear function corresponding to the key attribute.
  MFVec3f _keyValue;
};

}
}

#endif
