/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            TemplateMeshBuilderGeometry3DVisitor.h          //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEMESHBUILDERGEOMETRY3DVISITOR_H
#define TEMPLATEMESHBUILDERGEOMETRY3DVISITOR_H

#include "Geometry3DVisitor.h"
#include "TemplateMeshBuilderGlobalVariables.h"
#include "TemplateMeshBuilderTraits.h"
#include "Coordinate.h"
#include "Box.h"
#include "Cone.h"
#include "Cylinder.h"
#include "IndexedFaceSet.h"
#include "Sphere.h"
#include "MESH_TemplateMesh.h"

namespace X3DTK {
namespace X3D {

/// Visitor for the Geometry3D component of the TemplateMeshBuilder processor.

template<class MData, class VData, class EData, class FData>
class TemplateMeshBuilderGeometry3DVisitor : public Geometry3DVisitor
{
public:
  /// Constructor.
  TemplateMeshBuilderGeometry3DVisitor()
  : Geometry3DVisitor()
  {
    // Enter functions.
    defineNewEnterFunction(&TemplateMeshBuilderGeometry3DVisitor::enterIndexedFaceSet);

    // GlobalVariables assignation.
    globalVariables = GVManager::getInstanceOf<TemplateMeshBuilderGlobalVariables>();
  };
  
  /// Destructor.
  virtual ~TemplateMeshBuilderGeometry3DVisitor()
  {
  };

  /// Enters a IndexedFaceSet.
  virtual void enterIndexedFaceSet(IndexedFaceSet *I) const
  {
    // future optim? reserve the vertices and edges.
  
    typedef MESH::TemplateVertex<VData, EData, FData> Vertex;
    typedef MESH::MTemplateVertex<VData, EData, FData> MVertex;
    typedef MESH::TemplateEdge<EData, FData, VData> Edge;
    typedef MESH::MTemplateEdge<EData, FData, VData> MEdge;
    typedef MESH::TemplateFace<FData, VData, EData> Face;
    typedef MESH::MTemplateFace<FData, VData, EData> MFace;
           
    MESH::TemplateMesh<MData, VData, EData, FData> *TM = dynamic_cast<MESH::TemplateMesh<MData, VData, EData, FData> *>(globalVariables->getNode(I));
    if (TM == 0)
    {
      TM = new MESH::TemplateMesh<MData, VData, EData, FData>();
      globalVariables->addCoupleNode(I, TM);
    }
    globalVariables->pushNode(TM);
    
    // filling the mesh.
    // points creation
    Coordinate *CoordNode = dynamic_cast<Coordinate *>(I->getCoord());
    if (CoordNode == 0)
      return;
    
    MFVec3f pointArray = CoordNode->getPoint();
    if (pointArray.empty())
      return;
    
    TemplateMeshBuilderTraits<MData, VData, EData, FData, IndexedFaceSet *>::start(TM, I);
    
    // iterating the points
    for (MFVec3f::const_iterator itPoint = pointArray.begin(); itPoint != pointArray.end(); ++itPoint)
      TM->createVertex();
    
    SFBool ccw = I->getCcw();
    
    //iterating the faces
    const MFInt32 &coordIndex = I->getCoordIndex();
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    
    while (itCoordIndex != coordIndex.end())
    {
      Vertex *A, *B;      
      Edge *E;
      // creating the edges of the face.
      Vertex *start = TM->getVertices()[*itCoordIndex];
      
      A = start;
      ++itCoordIndex;

      std::vector<Edge *> faceEdgeList;      
      while (*itCoordIndex != -1)
      {
        B = TM->getVertices()[*itCoordIndex];
        ++itCoordIndex;
      
        E = TM->getEdge(A, B);
        if (E == 0)
          E = TM->createEdge(A, B);
            
        faceEdgeList.push_back(E);
        // moving to the next edge.
        A = B;
      }
      // ending the loop
      B = start;
      E = TM->getEdge(A, B);
      if (E == 0)
        E = TM->createEdge(A, B);
  
      faceEdgeList.push_back(E);
        
      // creating a new face.
      MEdge me(faceEdgeList, ccw);
      
      Face *F = TM->createFace(me);
      
      // processing the new face.
      TemplateMeshBuilderTraits<MData, VData, EData, FData, IndexedFaceSet *>::processNewFace(TM, I, F);

      ++itCoordIndex;
    }
    
    // ending the algorithm.
    TemplateMeshBuilderTraits<MData, VData, EData, FData, IndexedFaceSet *>::end(TM, I);
  };
  
protected:
  TemplateMeshBuilderGlobalVariables *globalVariables;
};

}
}

#endif
