/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DShapeNode.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DSHAPENODE_H
#define X3DSHAPENODE_H

#include "X3DTypes.h"
#include "X3DChildNode.h"

namespace X3DTK {
namespace X3D {

class X3DGeometryNode;
class X3DAppearanceNode;

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">Shape 
/// component</a> defining an X3DShapeNode
/// (<a href="../../doc/X3DSpecs/Part01/components/Shape.html#X3DShapeNode">
/// documentation</a>).

class X3DShapeNode : public X3DChildNode
{
public:
  /// Constructor.
  X3DShapeNode();
  /// Constructs an X3DShapeNode from its attributes.
  X3DShapeNode(const X3DGeometryNode *geometry, 
               const X3DAppearanceNode *appearance);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DShapeNode();
  
  /// Sets the Geometry attribute.
  void setGeometry(const X3DGeometryNode *geometry);
  /// Sets the Appearance attribute.
  void setAppearance(const X3DAppearanceNode *appearance);

  /// Gets the Geometry attribute.
  inline SFNode getGeometry() const {return _geometry;};
  /// Gets the Appearance attribute.
  inline SFNode getAppearance() const {return _appearance;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFAbstractNode &N);
  
protected:
  /// Copy constructor.	       
  X3DShapeNode(const X3DShapeNode &N);

private:  
  SFNode _geometry;
  SFNode _appearance;  
};

}
}

#endif
