#include "GL_Box.h"
#include "Box.h"
#include "BoxDrawArray.h"

namespace X3DTK {
namespace GL {

Box::Box()
: X3DGeometry3DNode(), _size(SFVec3f(0.0f, 0.0f, 0.0f)), _boxArray(0)
{
  defineTypeName("Box", "Geometry3D");
}

Box::Box(const Box &B)
: X3DGeometry3DNode(B), _size(B._size), _boxArray(0)
{
}

SFNode Box::clone() const
{
  return new Box(*this);
}

Box::~Box()
{
  _boxArray->removeInstance();
}

void Box::setSize(SFVec3f size)
{
  _size = size;
}

void Box::setBoxArray(BoxDrawArray *boxArray)
{
  _boxArray = boxArray;
}

void Box::update()
{
  if (x3dReference == 0)
    return;
    
  X3D::Box *B = static_cast<X3D::Box *>(x3dReference);

  _size = B->getSize();
  _boxArray = BoxDrawArray::getInstance();
}

void Box::render() const
{
  glEnable(GL_CULL_FACE);
  glFrontFace(GL_CW);
  glCullFace(GL_BACK);    
  glPushMatrix();
  
  glScalef(_size.x, _size.y, _size.z);  
  
  glInterleavedArrays(GL_N3F_V3F, 0, _boxArray->getBoxVertexArrayAddress());
  glDrawElements(GL_TRIANGLES, _boxArray->getBoxSize(), GL_UNSIGNED_INT, _boxArray->getBoxIndexArrayAddress());

  glPopMatrix();
  glDisable(GL_CULL_FACE);
}

}
}
