/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_StaticGroup.h                                //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef GLSTATICGROUP_H
#define GLSTATICGROUP_H

#include "GL_X3DChildNode.h"

namespace X3DTK {
namespace GL {

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// StaticGroup.

class StaticGroup : public X3DChildNode
{
public:
  /// Constructor.
  StaticGroup();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~StaticGroup();
  
  /// Adds a child to the Children attribute. Return TRUE if succeeds.
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child. Behaves like addChild.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child. Return TRUE if succeeds.
  virtual bool removeChild(const SFAbstractNode &N);
 
  /// Sets the children.
  void setChildren(const MFNode &children);
  
  /// Gets the Children attribute.
  inline MFNode getChildren() const {return _children;};

protected:  
  /// Copy constructor.		
  StaticGroup(const StaticGroup &G);
 
private:   
  /// Children attribute.
  MFNode _children;
};

}
}

#endif
