#include "GL_Transform.h"
#include "Transform.h"

using namespace std;

namespace X3DTK {
namespace GL {

Transform::Transform()
: X3DGroupingNode(), _transformMatrix(MFFloat(16))
{
  defineTypeName("Transform", "Grouping");
}

Transform::Transform(const Transform &T)
: X3DGroupingNode(T), _transformMatrix(T._transformMatrix) 
{
}

SFNode Transform::clone() const
{
  return new Transform(*this);
}

Transform::~Transform()
{
}

void Transform::setTransformMatrix(const MFFloat &transformMatrix)
{
  _transformMatrix = transformMatrix;
}

void Transform::update()
{
  if (x3dReference == 0)
    return;
    
  X3D::Transform *T = static_cast<X3D::Transform *>(x3dReference);
  SFMatrix34f transform(T->getTranslation(), T->getCenter(), T->getRotation(), T->getScaleOrientation(), T->getScale());
   
  float *tmp = transform.toFloat16();
  
  for (int i = 0; i < 16; ++i)
    _transformMatrix[i] = *(tmp + i);
}

void Transform::render() const
{
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glMultMatrixf(&_transformMatrix.front());
}

}
}
