#include "GL_Material.h"
#include "Material.h"

using namespace std;

namespace X3DTK {
namespace GL {

Material::Material()
: X3DMaterialNode(), _diffuseColor(MFFloat(4)), _ambientColor(MFFloat(4)), _emissiveColor(MFFloat(4)), _shininess(0.0f), _specularColor(MFFloat(4))
{
  defineTypeName("Material", "Shape");
}

Material::Material(const Material &M)
: X3DMaterialNode(M), _diffuseColor(M._diffuseColor), _ambientColor(M._ambientColor), _emissiveColor(M._emissiveColor), _shininess(M._shininess), _specularColor(M._specularColor)
{
}

SFNode Material::clone() const
{
  return new Material(*this);
}

Material::~Material()
{
}

void Material::setDiffuseColor(const MFFloat &diffuseColor)
{
  _diffuseColor = diffuseColor;
}

void Material::setAmbientColor(const MFFloat &ambientColor)
{
  _ambientColor = ambientColor;
}

void Material::setEmissiveColor(const MFFloat &emissiveColor)
{
  _emissiveColor = emissiveColor;
}

void Material::setShininess(SFFloat shininess)
{
  _shininess = shininess;
}

void Material::setSpecularColor(const MFFloat &specularColor)
{
  _specularColor = specularColor;
}

void Material::update()
{
  if (x3dReference == 0)
    return;
    
  X3D::Material *M = static_cast<X3D::Material *>(x3dReference);

  SFColor C;
  float t = 1.0f - M->getTransparency();
  float f = 1.0f - M->getAmbientIntensity();

  C = M->getDiffuseColor();
  
  _diffuseColor[0] = C.r;
  _diffuseColor[1] = C.g;
  _diffuseColor[2] = C.b;
  _diffuseColor[3] = t;

  _ambientColor[0] = f*C.r;
  _ambientColor[1] = f*C.g;
  _ambientColor[2] = f*C.b;
  _ambientColor[3] = t;

  C = M->getEmissiveColor();
  
  _emissiveColor[0] = C.r;
  _emissiveColor[1] = C.g;
  _emissiveColor[2] = C.b;
  _emissiveColor[3] = t;

  _shininess = 128.0f*M->getShininess();  

  C = M->getSpecularColor();
  
  _specularColor[0] = C.r;
  _specularColor[1] = C.g;
  _specularColor[2] = C.b;
  _specularColor[3] = t;
}

void Material::render() const
{
  glMaterialfv(GL_FRONT, GL_DIFFUSE, &_diffuseColor.front());
  glMaterialfv(GL_FRONT, GL_AMBIENT, &_ambientColor.front());
  glMaterialfv(GL_FRONT, GL_EMISSION, &_emissiveColor.front());
  glMaterialfv(GL_FRONT, GL_SHININESS, &_shininess); 
  glMaterialfv(GL_FRONT, GL_SPECULAR, &_specularColor.front());
}

}
}
