/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MeshTypes.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef MESHSCENEGRAPHTYPES_H
#define MESHSCENEGRAPHTYPES_H

#include "Platform.h"

#include <vector>
#include <list>
#include <iostream>

namespace X3DTK {
namespace MESH {

class X3DNode;
typedef X3DNode *SFNode;
typedef std::list<SFNode> MFNode;

template<class A, class B, class C> class TemplateVertex;
template<class A, class B, class C> class TemplateEdge;
template<class A, class B, class C> class TemplateFace;

/// Class providing a vector of Vertex for convenience, as well as object iterators.

template<class VData, class EData, class FData>
class MTemplateVertex : public std::vector<TemplateVertex<VData, EData, FData> *>
{
public:
  /// Constructor.
  MTemplateVertex() : std::vector<TemplateVertex<VData, EData, FData> *>() {};
  /// Constructor from a vector.
  explicit MTemplateVertex(const std::vector<TemplateVertex<VData, EData, FData> *> &vector)
  : std::vector<TemplateVertex<VData, EData, FData> *>(vector)
  {}; 
};

/// Class providing a vector of Edge for convenience, as well as object iterators.

template<class EData, class FData, class VData>
class MTemplateEdge : public std::vector<TemplateEdge<EData, FData, VData> *>
{
public:
  /// Constructor.
  MTemplateEdge() : std::vector<TemplateEdge<EData, FData, VData> *>() {};
  /// Constructor from a vector.
  explicit MTemplateEdge(const std::vector<TemplateEdge<EData, FData, VData> *> &vector, bool ccw)
  : std::vector<TemplateEdge<EData, FData, VData> *>(vector)
  {
    if (!ccw)
    {
      unsigned int last = vector.size() - 1;
      unsigned int mid = (last + 1)/2;
      for (unsigned int i = 0; i < mid; ++i)
      {
        TemplateEdge<EData, FData, VData> *temp = (*this)[i];
        (*this)[i] = (*this)[last - i];
        (*this)[last - i] = temp;
      }
    }  
  };
};

/// Class providing a vector of Face for convenience, as well as object iterators.

template<class FData, class VData, class EData>
class MTemplateFace : public std::vector<TemplateFace<FData, VData, EData> *>
{
public:
  /// Constructor.
  MTemplateFace() : std::vector<TemplateFace<FData, VData, EData> *>() {};
  /// Constructor from a vector.
  explicit MTemplateFace(const std::vector<TemplateFace<FData, VData, EData> *> &vector)
  : std::vector<TemplateFace<FData, VData, EData> *>(vector)
  {};
};

}
}

#endif
