#include "DFSNURBSWalker.h"
#include "NurbsSurface.h"
#include "TrimmedSurface.h"

namespace X3DTK {
namespace X3D {

DFSNURBSWalker::DFSNURBSWalker()
: X3DComponentWalker()
{
  defineComponentName("NURBS", "X3D");

  defineNewWalkingFunction(&DFSNURBSWalker::walkNurbsSurface);
  defineNewWalkingFunction(&DFSNURBSWalker::walkTrimmedSurface);
}

DFSNURBSWalker::~DFSNURBSWalker()
{
}

void DFSNURBSWalker::walkNurbsSurface(NurbsSurface *N) const
{
  visitor->enter(N);

  SFNode C = N->getTexCoord();
  if ((C != 0) && (visitor->walkOn(N, C)))
    walker->walk(C);

  visitor->leave(N);
}

void DFSNURBSWalker::walkTrimmedSurface(TrimmedSurface *T) const
{
  visitor->enter(T);

  MFNode trimmingContour = T->getTrimmingContour();
  for (MFNode::iterator it = trimmingContour.begin(); it != trimmingContour.end(); ++it)
    if ((*it != 0) && (visitor->walkOn(T, *it)))
      walker->walk(*it);

  SFNode C = T->getSurface();
  if ((C != 0) && (visitor->walkOn(T, C)))
    walker->walk(C);

  visitor->leave(T);
}

}
}
