#include "NurbsCurve.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

NurbsCurve::NurbsCurve()
  : X3DParametricGeometryNode(),
    _controlPoint(MFVec3f()),
    _tesselation (0),
    _weight      (MFDouble()),
    _knot        (MFDouble()),
    _order       (3)
{
  defineTypeName("NurbsCurve", "NURBS");
}

NurbsCurve::NurbsCurve(const MFVec3f  &controlPoint,
                       SFInt32        tesselation,
                       const MFDouble &weight,
                       const MFDouble &knot,
                       SFInt32        order)
  : X3DParametricGeometryNode(),
    _controlPoint(controlPoint),
    _tesselation (tesselation),
    _weight      (weight),
    _knot        (knot),
    _order       (order)
{
  defineTypeName("NurbsCurve", "NURBS");
}

NurbsCurve::NurbsCurve(const NurbsCurve &N)
  : X3DParametricGeometryNode(N),
    _controlPoint(N._controlPoint),
    _tesselation (N._tesselation),
    _weight      (N._weight),
    _knot        (N._knot),
    _order       (N._order)
{
}

SFNode NurbsCurve::clone() const
{
  return new NurbsCurve(*this);
}

NurbsCurve::~NurbsCurve()
{
}

void NurbsCurve::setControlPoint(const MFVec3f &controlPoint)
{
  _controlPoint = controlPoint;
}

void NurbsCurve::setTesselation(SFInt32 tesselation)
{
  _tesselation = tesselation;
}

void NurbsCurve::setWeight(const MFDouble &weight)
{
  _weight = weight;
}

void NurbsCurve::setKnot(const MFDouble &knot)
{
  _knot = knot;
}

void NurbsCurve::setOrder(SFInt32 order)
{
  _order = order;
}

void NurbsCurve::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("controlPoint");
  if (index != -1)
    _controlPoint = MFVec3f(element->getAttribute(index));

  index = element->getIndexAttribute("tesselation");
  if (index != -1)
    _tesselation = element->getAttribute(index).toInt();

  index = element->getIndexAttribute("weight");
  if (index != -1)
    _weight = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("knot");
  if (index != -1)
    _knot = MFDouble(element->getAttribute(index));

  index = element->getIndexAttribute("order");
  if (index != -1)
    _order = element->getAttribute(index).toInt();
}

SFString NurbsCurve::write() const
{
  SFString attr;
  if (!_controlPoint.empty())
    attr += " controlPoint=\"" + toSFString(_controlPoint) + "\"";
  if (_tesselation != 0)
    attr += " tesselation=\"" + toSFString(_tesselation) + "\"";
  if (!_weight.empty())
    attr += " weight=\"" + toSFString(_weight) + "\"";
  if (!_knot.empty())
    attr += " knot=\"" + toSFString(_knot) + "\"";
  if (_order != 3)
    attr += " order=\"" + toSFString(_order) + "\"";

  return attr;
}


}
}
