#include "TrimmedSurface.h"
#include "Contour2D.h"
#include "NurbsSurface.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TrimmedSurface::TrimmedSurface()
  : X3DParametricGeometryNode(),
    _surface(0)
{
  defineTypeName("TrimmedSurface", "NURBS");
}

TrimmedSurface::TrimmedSurface(const NurbsSurface *surface)
  : X3DParametricGeometryNode(),
    _surface((SFNode)surface)
{
  defineTypeName("TrimmedSurface", "NURBS");
}

TrimmedSurface::TrimmedSurface(const TrimmedSurface &N)
  : X3DParametricGeometryNode(N),
    _trimmingContour(N._trimmingContour),
    _surface        (N._surface)
{
}

SFNode TrimmedSurface::clone() const
{
  return new TrimmedSurface(*this);
}

TrimmedSurface::~TrimmedSurface()
{
  removeParentFromChild(this, _surface);
}

void TrimmedSurface::setSurface(const NurbsSurface *surface)
{
  removeParentFromChild(this, _surface);
  _surface = (SFNode)surface;
  addParentToChild(this, _surface);
}

bool TrimmedSurface::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<Contour2D *>(N) != 0)
    {
      _trimmingContour.push_back(static_cast<SFNode>(N));
      addParentToChild(this, N);
      return true;
    }
  
  if (dynamic_cast<NurbsSurface *>(N) != 0)
    {
      if (_surface != 0)
        return false;
      
      setSurface(static_cast<NurbsSurface *>(N));
      return true;    
    }

  cerr << "TrimmedSurface::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;
}

bool TrimmedSurface::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<Contour2D *>(N) != 0)
    {
      _trimmingContour.push_back(static_cast<SFNode>(N));
      addParentToChild(N, this);
      return true;
    }

  if (dynamic_cast<NurbsSurface *>(N) != 0)
    {
      setSurface(static_cast<NurbsSurface *>(N));
      return true;    
    }

  cerr << "TrimmedSurface::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;    
}

bool TrimmedSurface::removeChild(const SFAbstractNode &N)
{
  MFNode::iterator res = find(_trimmingContour.begin(), _trimmingContour.end(), N);
  if (res != _trimmingContour.end())
  {
    _trimmingContour.erase(res);
    removeParentFromChild(N, this);
    return true;
  }

  if (_surface == N)
  {
    setSurface(0);
    return true;
  } 

  return false;
}

}
}
