/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            PointSet.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef POINTSET_H
#define POINTSET_H

#include "X3DTypes.h"
#include "X3DGeometryNode.h"

namespace X3DTK {
namespace X3D {

class X3DColorNode;
class X3DCoordinateNode;

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
/// Rendering component</a> defining a PointSet
/// (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#PointSet">
/// documentation</a>).
class PointSet : public X3DGeometryNode
{
public:
  /// Constructor.
  PointSet();
  /// Constructs a PointSet from its attributes.
  PointSet(const X3DColorNode *color,
           const X3DCoordinateNode *coord);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~PointSet();
  
  /// Sets the Color attribute.
  void setColor(const X3DColorNode *color);
  /// Sets the Coord attribute.
  void setCoord(const X3DCoordinateNode *coord);
  
  /// Gets the Color attribute.
  inline SFNode getColor() const {return _color;};
  /// Gets the Coord attribute.
  inline SFNode getCoord() const {return _coord;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFAbstractNode &N);

protected:
  /// Copy constructor. 
  PointSet(const PointSet &P);
  
private:  
  /// Color node.
  SFNode _color; 
  /// Coord node. 
  SFNode _coord;  
};

}
}

#endif
