#include "TriangleFanSet.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TriangleFanSet::TriangleFanSet()
: X3DComposedGeometryNode(), _convex(true), _creaseAngle(0.0f)
{
  defineTypeName("TriangleFanSet", "Rendering");
}

TriangleFanSet::TriangleFanSet(const X3DColorNode *color, const X3DCoordinateNode *coord, const X3DNormalNode *normal, const X3DTextureCoordinateNode *texCoord, SFBool ccw, SFBool colorPerVertex, SFBool convex, SFFloat creaseAngle, const MFInt32 &fanCount, SFBool normalPerVertex, SFBool solid)
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
_convex(convex), _creaseAngle(creaseAngle), _fanCount(fanCount)
{
  defineTypeName("TriangleFanSet", "Rendering");
}

           
TriangleFanSet::TriangleFanSet(const TriangleFanSet &T)
: X3DComposedGeometryNode(T), _convex(T._convex), _creaseAngle(T._creaseAngle), _fanCount(T._fanCount)
{
}

SFNode TriangleFanSet::clone() const
{
  return new TriangleFanSet(*this);
}

TriangleFanSet::~TriangleFanSet()
{
}

void TriangleFanSet::setConvex(SFBool convex)
{
  _convex = convex;
}

void TriangleFanSet::setCreaseAngle(SFFloat creaseAngle)
{
  _creaseAngle = creaseAngle;
}

void TriangleFanSet::setFanCount(const MFInt32 &fanCount)
{
  _fanCount  = fanCount;
}

void TriangleFanSet::load(const X3DFileElement *element)
{
  X3DComposedGeometryNode::load(element);
  
  int index;
  index = element->getIndexAttribute("convex");  
  if (index != -1)
    _convex = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    _creaseAngle = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("fanCount");
  if (index != -1)
    _fanCount = MFInt32(element->getAttribute(index));  
}

SFString TriangleFanSet::write() const
{
  SFString attr;
  attr = X3DComposedGeometryNode::write();
  
  if (!_convex)  
    attr += " convex=\"FALSE\"";
  if (_creaseAngle != 0.0f)
    attr += " creaseAngle=\"" + toSFString(_creaseAngle) + "\"";
  if (!_fanCount.empty())
    attr += " fanCount=\"" + toSFString(_fanCount) + "\"";
  
  return attr;
}

}
}
