/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            TriangleFanSet.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef TRIANGLEFANSET_H
#define TRIANGLEFANSET_H

#include "X3DTypes.h"
#include "X3DComposedGeometryNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
/// Rendering component</a> defining a TriangleFanSet
/// (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#TriangleFanSet">
/// documentation</a>).

class TriangleFanSet : public X3DComposedGeometryNode
{
public:
  /// Constructor.
  TriangleFanSet();
  /// Constructs a triangleFanSet from its attributes.
  TriangleFanSet(const X3DColorNode *color,
                 const X3DCoordinateNode *coord,
                 const X3DNormalNode *normal,
                 const X3DTextureCoordinateNode *texCoord,
                 SFBool ccw,
                 SFBool colorPerVertex,
                 SFBool convex,
                 SFFloat creaseAngle,
                 const MFInt32 &fanCount,
                 SFBool normalPerVertex,
                 SFBool solid);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~TriangleFanSet();
  
  /// Sets the Convex attribute.
  void setConvex(SFBool convex);
  /// Sets the CreaseAngle attribute.
  void setCreaseAngle(SFFloat creaseAngle);
  /// Sets the FanCount attribute.
  void setFanCount(const MFInt32 &fanCount);
  
  /// Gets the Convex attribute.
  inline SFBool getConvex() const {return _convex;};
  /// Gets the CreaseAngle attribute.
  inline SFFloat getCreaseAngle() const {return _creaseAngle;};
  /// Gets the FanCount attribute.
  inline const MFInt32 &getFanCount() const {return _fanCount;};

  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.         
  TriangleFanSet(const TriangleFanSet &T);

private:
  /// Convex attribute.
  SFBool _convex;  
  /// Crease angle attribute.
  SFFloat _creaseAngle;
  /// Fan count attribute.
  MFInt32 _fanCount; 
};

}
}

#endif
