#include <vector>
#include "MeshNormalUpdaterCoreVisitor.h"

using namespace std;

namespace X3DTK {
namespace MESH {

MeshNormalUpdaterCoreVisitor::MeshNormalUpdaterCoreVisitor()
{
  defineNewEnterFunction(&MeshNormalUpdaterCoreVisitor::enterMesh);
}

void MeshNormalUpdaterCoreVisitor::enterMesh(Mesh *M) const
{
  bool normalPerVertex = M->data().getNormalPerVertex();
  
  vector<SFVec3f> normals;
  
  // Compute all faces' normals 
  const Mesh::MFace& faces = M->getFaces();
  for (Mesh::MFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
    {
      SFVec3f normal;
      const Mesh::MEdge& edges = (*f)->getEdges();
      Mesh::MEdge::const_iterator e = edges.begin();
      // Choose a pivot point
      const SFPoint3f &firstPoint = (*e)->getFromVertex()->data().getPoint();
	  
      // Initialize secondPoint from second edge
      ++e;
      SFPoint3f secondPoint = (*e)->getFromVertex()->data().getPoint();

      // Start loop at third edge
      ++e;
      for (Mesh::MEdge::const_iterator end = edges.end() ; e != end; ++e)
	{
	  SFPoint3f thirdPoint = (*e)->getFromVertex()->data().getPoint();
	  normal = normal + crossprod( (secondPoint-firstPoint), (thirdPoint-firstPoint) );
	  secondPoint = thirdPoint;
	}
      normals.push_back(normal);

      // Update face normal (if face does not already has a normal)
      if ((!normalPerVertex) && ((*f)->data().getNormal().norm() < 1E-4))
	  (*f)->data().setNormal(normal.normalize());
    }

  if (normalPerVertex)
    {
      const Mesh::MVertex& vertices = M->getVertices();
      for (Mesh::MVertex::const_iterator v = vertices.begin(), end = vertices.end() ; v != end; ++v)
	{
	  SFVec3f normal;

	  // Compute point normal as an area weighted average of faces' normals
	  const Mesh::MFace& faces = (*v)->getFaces();
	  for (Mesh::MFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
	    normal = normal + normals[(*f)->getIndex()];

	  normal.normalize();

	  // Store computed normal in VertexFData
	  for (Mesh::MFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
	    if ((*v)->data().getNormalOfFace(*f).norm() < 1E-4)
	      (*v)->data().setNormalOfFace(normal, *f);
	}
    }
}

} 
} 
