#include "BBoxUpdaterGeometry3DVisitor.h"
#include "Box.h"
#include "Cone.h"
#include "Cylinder.h"
#include "Sphere.h"
#include "BBox.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

BBoxUpdaterGeometry3DVisitor::BBoxUpdaterGeometry3DVisitor()
: Geometry3DVisitor()
{
  // Enter functions.
  defineNewEnterFunction(&BBoxUpdaterGeometry3DVisitor::enterBox);
  defineNewEnterFunction(&BBoxUpdaterGeometry3DVisitor::enterCone);
  defineNewEnterFunction(&BBoxUpdaterGeometry3DVisitor::enterCylinder);
  defineNewEnterFunction(&BBoxUpdaterGeometry3DVisitor::enterSphere);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<BBoxUpdaterGlobalVariables>();
}

BBoxUpdaterGeometry3DVisitor::~BBoxUpdaterGeometry3DVisitor()
{
}

void BBoxUpdaterGeometry3DVisitor::enterBox(Box *B) const
{
  BBox *BB = globalVariables->getBBox(B);
  if (BB == 0)
  {
    BB = new BBox(SFVec3f(0.0f, 0.0f, 0.0f), B->getSize());
    globalVariables->addBBox(B, BB);
    globalVariables->setShapeBBox(*BB);  
  }
}

void BBoxUpdaterGeometry3DVisitor::enterCone(Cone *C) const
{
  BBox *BB = globalVariables->getBBox(C);
  if (BB == 0)
  {
    BB = new BBox(SFVec3f(0.0f, 0.0f, 0.0f), SFVec3f(2.0f*C->getBottomRadius(), C->getHeight(), 2.0f*C->getBottomRadius()));
    globalVariables->addBBox(C, BB); 
    globalVariables->setShapeBBox(*BB);   
  }
}

void BBoxUpdaterGeometry3DVisitor::enterCylinder(Cylinder *C) const
{
  BBox *BB = globalVariables->getBBox(C);
  if (BB == 0)
  {
    BB = new BBox(SFVec3f(0.0f, 0.0f, 0.0f), SFVec3f(2.0f*C->getRadius(), C->getHeight(), 2.0f*C->getRadius()));
    globalVariables->addBBox(C, BB);
    globalVariables->setShapeBBox(*BB);    
  }
}

void BBoxUpdaterGeometry3DVisitor::enterSphere(Sphere *S) const
{
  BBox *BB = globalVariables->getBBox(S);
  if (BB == 0)
  {
    BB = new BBox(SFVec3f(0.0f, 0.0f, 0.0f), SFVec3f(2.0f*S->getRadius(), 2.0f*S->getRadius(), 2.0f*S->getRadius()));
    globalVariables->addBBox(S, BB);
    globalVariables->setShapeBBox(*BB);    
  }
}

}
}
