#include "GLBuilder.h"
#include "DFSGraphTraversal.h"
#include "GLBuilderCoreVisitor.h"
#include "GLBuilderGeometry3DVisitor.h"
#include "GLBuilderGroupingVisitor.h"
#include "GLBuilderRenderingVisitor.h"
#include "GLBuilderShapeVisitor.h"
#include "GLBuilderTexturingVisitor.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

GLBuilder::GLBuilder()
: X3DGLBuilder()
{
  globalVariables = GVManager::getInstanceOf<GLBuilderGlobalVariables>();
  graphTraversal = new DFSGraphTraversal();
  
  graphTraversal->setComponentVisitor(new GLBuilderCoreVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderGeometry3DVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderGroupingVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderRenderingVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderShapeVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderTexturingVisitor());
}

GLBuilder::~GLBuilder()
{
  GVManager::removeInstanceOf<GLBuilderGlobalVariables>();
  
  delete graphTraversal;
}

GL::SFNode GLBuilder::build(SFNode N)
{
  globalVariables->init();
  graphTraversal->traverse(N);
  globalVariables->finish();
  
  return globalVariables->getRoot();
}

GL::Scene *GLBuilder::build(Scene *S)
{
  return static_cast<GL::Scene *>(build(static_cast<SFNode>(S)));
}

}
}
