#include "GLBuilderCoreVisitor.h"
#include "Scene.h"
#include "X3DNode.h"
#include "GL_Scene.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

GLBuilderCoreVisitor::GLBuilderCoreVisitor()
: CoreVisitor()
{
  // Enter functions.
  defineNewEnterFunction(&GLBuilderCoreVisitor::enterScene);
  defineNewEnterFunction(&GLBuilderCoreVisitor::enterX3DNode);

  // WalkOn functions.
  defineNewWalkOnFunction(&GLBuilderCoreVisitor::walkOnX3DNode);
  
  // Leave functions.
  defineNewLeaveFunction(&GLBuilderCoreVisitor::leaveX3DNode);

  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLBuilderGlobalVariables>();
}

GLBuilderCoreVisitor::~GLBuilderCoreVisitor()
{
}

void GLBuilderCoreVisitor::enterScene(Scene *S) const
{
  GL::SFNode GS = globalVariables->getNode(S);
  if (GS == 0)
  {
    GS = new GL::Scene();
    GS->setX3DReference(S);
    globalVariables->addCoupleNode(S, GS);
  }
  globalVariables->pushNode(GS);  
}

void GLBuilderCoreVisitor::enterX3DNode(X3DNode *N) const
{
  cerr << "warning: no GL implementation for " << N->getTypeName() << " !" << endl;
  globalVariables->pushNode(0);
}

bool GLBuilderCoreVisitor::walkOnX3DNode(X3DNode *N, SFAbstractNode child) const
{
  return (globalVariables->getTop() != 0);
}

void GLBuilderCoreVisitor::leaveX3DNode(X3DNode *) const
{
  GL::SFNode N = globalVariables->getTop();
  if (N != 0)
    N->update();
    
  globalVariables->popNode();
}

}
}
