#include "GLBuilderGeometry3DVisitor.h"
#include "Box.h"
#include "Cone.h"
#include "Cylinder.h"
#include "IndexedFaceSet.h"
#include "Sphere.h"
#include "GL_Box.h"
#include "GL_Cone.h"
#include "GL_Cylinder.h"
#include "GL_IndexedFaceSet.h"
#include "GL_Sphere.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

GLBuilderGeometry3DVisitor::GLBuilderGeometry3DVisitor()
: Geometry3DVisitor()
{
  // Enter functions.
  defineNewEnterFunction(&GLBuilderGeometry3DVisitor::enterBox);
  defineNewEnterFunction(&GLBuilderGeometry3DVisitor::enterCone);
  defineNewEnterFunction(&GLBuilderGeometry3DVisitor::enterCylinder);
  defineNewEnterFunction(&GLBuilderGeometry3DVisitor::enterIndexedFaceSet);
  defineNewEnterFunction(&GLBuilderGeometry3DVisitor::enterSphere);

  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLBuilderGlobalVariables>();
}

GLBuilderGeometry3DVisitor::~GLBuilderGeometry3DVisitor()
{
}

void GLBuilderGeometry3DVisitor::enterBox(Box *B) const
{
  GL::SFNode GB = globalVariables->getNode(B);
  if (GB == 0)
  {
    GB = new GL::Box();
    GB->setX3DReference(B);
    globalVariables->addCoupleNode(B, GB);
  }
  globalVariables->pushNode(GB);
}
  
void GLBuilderGeometry3DVisitor::enterCone(Cone *C) const
{
  GL::SFNode GC = globalVariables->getNode(C);
  if (GC == 0)
  {
    GC = new GL::Cone();
    GC->setX3DReference(C);
    globalVariables->addCoupleNode(C, GC);
  }
  globalVariables->pushNode(GC);
}

void GLBuilderGeometry3DVisitor::enterCylinder(Cylinder *C) const
{
  GL::SFNode GC = globalVariables->getNode(C);
  if (GC == 0)
  {
    GC = new GL::Cylinder();
    GC->setX3DReference(C);
    globalVariables->addCoupleNode(C, GC);
  }
  globalVariables->pushNode(GC);
}

void GLBuilderGeometry3DVisitor::enterIndexedFaceSet(IndexedFaceSet *I) const
{
  GL::SFNode GI = globalVariables->getNode(I);
  if (GI == 0)
  {
    GI = new GL::IndexedFaceSet();
    GI->setX3DReference(I);
    globalVariables->addCoupleNode(I, GI);
  }
  globalVariables->pushNode(GI);

}
void GLBuilderGeometry3DVisitor::enterSphere(Sphere *S) const
{
  GL::SFNode GS = globalVariables->getNode(S);
  if (GS == 0)
  {
    GS = new GL::Sphere();
    GS->setX3DReference(S);
    globalVariables->addCoupleNode(S, GS);
  }
  globalVariables->pushNode(GS);
}

}
}
