#include "GLBuilderRenderingVisitor.h"
#include "IndexedLineSet.h"
#include "PointSet.h"
#include "TriangleFanSet.h"
#include "TriangleSet.h"
#include "TriangleStripSet.h"
#include "GL_IndexedLineSet.h"
#include "GL_PointSet.h"
#include "GL_TriangleFanSet.h"
#include "GL_TriangleSet.h"
#include "GL_TriangleStripSet.h"
#include "X3DGeometryNode.h"

namespace X3DTK {
namespace X3D {

GLBuilderRenderingVisitor::GLBuilderRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  defineNewEnterFunction(&GLBuilderRenderingVisitor::enterIndexedLineSet);
  defineNewEnterFunction(&GLBuilderRenderingVisitor::enterPointSet);
  defineNewEnterFunction(&GLBuilderRenderingVisitor::enterTriangleFanSet);
  defineNewEnterFunction(&GLBuilderRenderingVisitor::enterTriangleSet);
  defineNewEnterFunction(&GLBuilderRenderingVisitor::enterTriangleStripSet);

  // WalkOn functions
  defineNewWalkOnFunction(&GLBuilderRenderingVisitor::walkOnX3DGeometryNode);
  
  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLBuilderGlobalVariables>();
}

GLBuilderRenderingVisitor::~GLBuilderRenderingVisitor()
{
}

bool GLBuilderRenderingVisitor::walkOnX3DGeometryNode(X3DGeometryNode *, SFAbstractNode) const
{
  return false;
}

void GLBuilderRenderingVisitor::enterIndexedLineSet(IndexedLineSet *I) const
{
  GL::SFNode GI = globalVariables->getNode(I);
  if (GI == 0)
  {
    GI = new GL::IndexedLineSet();
    GI->setX3DReference(I);
    globalVariables->addCoupleNode(I, GI);
  }   
  globalVariables->pushNode(GI);
}

void GLBuilderRenderingVisitor::enterPointSet(PointSet *P) const
{
  GL::SFNode GP = globalVariables->getNode(P);
  if (GP == 0)
  {
    GP = new GL::PointSet();
    GP->setX3DReference(P);
    globalVariables->addCoupleNode(P, GP);
  }   
  globalVariables->pushNode(GP);
}

void GLBuilderRenderingVisitor::enterTriangleFanSet(TriangleFanSet *T) const
{
  GL::SFNode GT = globalVariables->getNode(T);
  if (GT == 0)
  {
    GT = new GL::TriangleFanSet();
    GT->setX3DReference(T);
    globalVariables->addCoupleNode(T, GT);
  }   
  globalVariables->pushNode(GT);
}

void GLBuilderRenderingVisitor::enterTriangleSet(TriangleSet *T) const
{
  GL::SFNode GT = globalVariables->getNode(T);
  if (GT == 0)
  {
    GT = new GL::TriangleSet();
    GT->setX3DReference(T);
    globalVariables->addCoupleNode(T, GT);
  }   
  globalVariables->pushNode(GT);
}

void GLBuilderRenderingVisitor::enterTriangleStripSet(TriangleStripSet *T) const
{
  GL::SFNode GT = globalVariables->getNode(T);
  if (GT == 0)
  {
    GT = new GL::TriangleStripSet();
    GT->setX3DReference(T);
    globalVariables->addCoupleNode(T, GT);
  }   
  globalVariables->pushNode(GT);
}

}
}
