#include "GLBuilderShapeVisitor.h"
#include "Appearance.h"
#include "FillProperties.h"
#include "LineProperties.h"
#include "Material.h"
#include "Shape.h"
#include "GL_Appearance.h"
#include "GL_FillProperties.h"
#include "GL_LineProperties.h"
#include "GL_Material.h"
#include "GL_Shape.h"

namespace X3DTK {
namespace X3D {

GLBuilderShapeVisitor::GLBuilderShapeVisitor()
: ShapeVisitor()
{
  // Enter functions.
  defineNewEnterFunction(&GLBuilderShapeVisitor::enterAppearance);
  defineNewEnterFunction(&GLBuilderShapeVisitor::enterFillProperties);
  defineNewEnterFunction(&GLBuilderShapeVisitor::enterLineProperties);
  defineNewEnterFunction(&GLBuilderShapeVisitor::enterMaterial);
  defineNewEnterFunction(&GLBuilderShapeVisitor::enterShape);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLBuilderGlobalVariables>();
}

GLBuilderShapeVisitor::~GLBuilderShapeVisitor()
{
}

void GLBuilderShapeVisitor::enterAppearance(Appearance *A) const
{
  GL::SFNode GA = globalVariables->getNode(A);
  if (GA == 0)
  {
    GA = new GL::Appearance();
    GA->setX3DReference(A);
    globalVariables->addCoupleNode(A, GA);
  }   
  globalVariables->pushNode(GA);
}

void GLBuilderShapeVisitor::enterFillProperties(FillProperties *F) const
{
  GL::SFNode GF = globalVariables->getNode(F);
  if (GF == 0)
  {
    GF = new GL::FillProperties();
    GF->setX3DReference(F);
    globalVariables->addCoupleNode(F, GF);
  }   
  globalVariables->pushNode(GF);
}

void GLBuilderShapeVisitor::enterLineProperties(LineProperties *L) const
{
  GL::SFNode GL = globalVariables->getNode(L);
  if (GL == 0)
  {
    GL = new GL::LineProperties();
    GL->setX3DReference(L);
    globalVariables->addCoupleNode(L, GL);
  }   
  globalVariables->pushNode(GL);
}

void GLBuilderShapeVisitor::enterMaterial(Material *M) const
{
  GL::SFNode GM = globalVariables->getNode(M);
  if (GM == 0)
  {
    GM = new GL::Material();
    GM->setX3DReference(M);
    globalVariables->addCoupleNode(M, GM);
  }   
  globalVariables->pushNode(GM);
}

void GLBuilderShapeVisitor::enterShape(Shape *S) const
{
  GL::SFNode GS = globalVariables->getNode(S);
  if (GS == 0)
  {
    GS = new GL::Shape();
    GS->setX3DReference(S);
    globalVariables->addCoupleNode(S, GS);
  }   
  globalVariables->pushNode(GS);
}

}
}
