#include "MeshBuilder.h"
#include "IndexedFaceSet.h"
#include "Coordinate.h"
#include "Normal.h"
#include "TextureCoordinate.h"
#include "Color.h"
#include "ColorRGBA.h"

namespace X3DTK {
namespace X3D {

void TemplateMeshBuilderTraits<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData, IndexedFaceSet *>::start(MESH::TemplateMesh<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData> *mesh, IndexedFaceSet *I) 
{
  // normals
  mesh->data().setNormal(I->getNormal() != 0);
  mesh->data().setNormalPerVertex(I->getNormalPerVertex());
  
  // colors
  mesh->data().setColor(I->getColor() != 0);
  mesh->data().setColorPerVertex(I->getColorPerVertex());
  
  // texCoords
  mesh->data().setTexCoord(I->getTexCoord() != 0);
}

void TemplateMeshBuilderTraits<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData, IndexedFaceSet *>::processNewFace(MESH::TemplateMesh<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData> *mesh, IndexedFaceSet *I, MESH::TemplateFace<MESH::FaceData, MESH::VertexData, MESH::EdgeData> *F)
{
}

void TemplateMeshBuilderTraits<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData, IndexedFaceSet *>::end(MESH::TemplateMesh<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData> *mesh, IndexedFaceSet *I) 
{
  MFVec3f pointArray;
  MFVec3f normalArray;
  MFColorRGBA colorArray;
  MFVec2f texCoordArray;
  MFInt32 coordIndex;
  MFInt32 normalIndex;
  MFInt32 colorIndex;
  MFInt32 texCoordIndex;

  X3D::Coordinate *CoordinateNode = dynamic_cast<X3D::Coordinate *>(I->getCoord());
  if (CoordinateNode != 0)
    pointArray = CoordinateNode->getPoint();

  X3D::Normal *NormalNode = dynamic_cast<X3D::Normal *>(I->getNormal());
  if (NormalNode != 0)
    normalArray = NormalNode->getVector();
  
  if (I->getColor() != 0)
  {
    if (I->getColor()->getTypeName() == "ColorRGBA")
      colorArray = dynamic_cast<X3D::ColorRGBA *>(I->getColor())->getColor();
    else  
      colorArray = dynamic_cast<X3D::Color *>(I->getColor())->getColor();
  }
  
  X3D::TextureCoordinate *TexCoordNode = dynamic_cast<X3D::TextureCoordinate *>(I->getTexCoord());
  if (TexCoordNode != 0)
    texCoordArray = TexCoordNode->getPoint();
  
  coordIndex = I->getCoordIndex();
  colorIndex = I->getColorIndex();
  normalIndex = I->getNormalIndex();
  texCoordIndex = I->getTexCoordIndex();
      
  SFBool normalPerVertex = I->getNormalPerVertex();
  SFBool colorPerVertex = I->getColorPerVertex();
  
  MFInt32::const_iterator itCoordIndex = coordIndex.begin();
  MFInt32::const_iterator itColorIndex;
  MFInt32::const_iterator itNormalIndex;
  MFInt32::const_iterator itTexCoordIndex;
  
  if (colorIndex.empty())
    itColorIndex = coordIndex.begin();
  else
    itColorIndex = colorIndex.begin();  
  
  if (normalIndex.empty())
    itNormalIndex = coordIndex.begin();
  else
    itNormalIndex = normalIndex.begin();  
  
  if (texCoordIndex.empty())
    itTexCoordIndex = coordIndex.begin();
  else
    itTexCoordIndex = texCoordIndex.begin();
    
  MESH::Mesh::MFace::const_iterator itFace = mesh->getFaces().begin(); 
  
  while (itCoordIndex != coordIndex.end())
  {
    while ((*itCoordIndex != -1) && (itCoordIndex != coordIndex.end()))
    {
      MESH::Mesh::Vertex *V = mesh->getVertices()[*itCoordIndex];
     
      if (!pointArray.empty())
        V->data().setPoint(SFPoint3f(pointArray[*itCoordIndex]));
      if (!normalArray.empty())
        V->data().setNormalOfFace(normalArray[*itNormalIndex], *itFace);
      if (!colorArray.empty())
        V->data().setColorOfFace(colorArray[*itColorIndex], *itFace);
      if (!texCoordArray.empty())
        V->data().setTexCoordOfFace(texCoordArray[*itTexCoordIndex], *itFace);  
      
      if (!normalPerVertex)
        (*itFace)->data().setNormal(normalArray[*itNormalIndex]);
      if (!colorPerVertex)
        (*itFace)->data().setColor(colorArray[*itColorIndex]);  
      
      ++itCoordIndex;
      if (colorPerVertex)
        ++itColorIndex;     
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itTexCoordIndex;
    }
    
    ++itCoordIndex;
    if (!colorPerVertex)
      ++itColorIndex;
    ++itNormalIndex;
    ++itTexCoordIndex;
    
    ++itFace;
  }
}
  

}
}
