#include "SceneSaver.h"
#include "DFSGraphTraversal.h"
#include "SceneSaverCoreVisitor.h"

#include <fstream>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

SceneSaver::SceneSaver()
: X3DSceneSaver()
{
  globalVariables = GVManager::getInstanceOf<SceneSaverGlobalVariables>();

  graphTraversal = new DFSGraphTraversal();  
  graphTraversal->setComponentVisitor(new SceneSaverCoreVisitor());
}

SceneSaver::~SceneSaver()
{
  GVManager::removeInstanceOf<SceneSaverGlobalVariables>();

  delete graphTraversal;
}

void SceneSaver::saveAs(Scene *S, const SFString &fileName) const
{
  if (S == 0)
  {
    cerr << "unable to write to file, because the Scene is NULL!" << endl;
    return;
  }
    
  ofstream output(fileName);
  
  output << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
  output << "<X3D>" << endl;
  
  graphTraversal->traverse(S);
  MFString text = globalVariables->getOutput();
  
  for (MFString::const_iterator it = text.begin(); it != text.end(); ++it)
    output << (*it) << endl;

  output << "</X3D>" << endl;
}

void SceneSaver::save(Scene *S) const
{
  saveAs(S, S->getFileName());
}

}
}
