/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            TemplateMeshBuilderGlobalVariables.h            //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEMESHBUILDERGLOBALVARIABLES_H
#define TEMPLATEMESHBUILDERGLOBALVARIABLES_H

#include "GlobalVariables.h"
#include "X3DNode.h"
#include "MESH_X3DNode.h"

#include <map>

namespace X3DTK {
namespace X3D {

/// GlobalVariables for the TemplateMeshBuilder processor.

class TemplateMeshBuilderGlobalVariables : public GlobalVariables
{
public:
  /// Constructor.
  TemplateMeshBuilderGlobalVariables();
  /// Destructor.
  virtual ~TemplateMeshBuilderGlobalVariables();
  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  virtual void finish();
  
  /// Adds the couple of nodes.
  void addCoupleNode(SFNode N, MESH::SFNode MN);
  /// Pushes the node.
  void pushNode(MESH::SFNode N);
  /// Pops the node.
  void popNode();
    
  /// Gets the GLScene.
  inline MESH::SFNode getRoot() const {return _root;};
  /// Gets the top of the stack.
  inline MESH::SFNode getTop() const {return _nodeStack.front();};
  /// Gets the MESH::SFNode related with N. Returns 0 if N not found.
  MESH::SFNode getNode(SFNode N) const;
  
private:
  MESH::SFNode _root;
  MESH::MFNode _nodeStack;
  std::map<SFNode, MESH::SFNode> _nodeCoupleMap; 
};

}
}

#endif
