/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            Component.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef COMPONENT_H
#define COMPONENT_H

#include "X3DTypes.h"
#include "SceneGraph.h"

namespace X3DTK {

class SceneGraph;

/// Class representing the concept of X3D component. It is a partition of the X3D 
/// nodes. It is implemented by a singleton per name.

class Component
{
  friend class Type;
  friend class X3DComponent;
  friend class Walker;
  friend class Visitor;
  
public:
  /// Gets the component of name name.
  static Component *getComponent(const SFString &sceneGraph, const SFString &name);
  
  /// Returns the name of the component.
  inline SFString getName() const {return _name;};
  /// Returns the name of the sceneGraph.
  inline SFString getSceneGraphName() const {return _sceneGraph->getName();};
  /// Gets the list of the types.
  inline MFType getTypeList() const {return _typeList;};
  /// Gets the scene graph.
  inline SceneGraph *getSceneGraph() const {return _sceneGraph;};
  
private:  
  Component(const SFString &name, const SFString &sceneGraph);
  ~Component();
  
  /// Adds one type.
  void addType(Type *type);
  /// Adds one reference.
  void addOneReference();
  /// Removes one reference of the Component.
  static void removeOneReference(Component *component);
  /// Computes the key.
  static SFString encodeKey(const SFString &sceneGraph, const SFString &name);
  /// Returns the encoded name of the component.
  inline SFString getEncodedName() const {return encodeKey(getSceneGraphName(), getName());};
    
  SFString _name;    
  int _refCount;
  MFType _typeList;
  SceneGraph *_sceneGraph;
  
  static MFComponent _componentList;
};

}

#endif
