#include "NameDict.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace X3DTK {

NameDict::NameDict()
{
}

NameDict::~NameDict()
{
  _nameDict.clear();
}
  
void NameDict::insert(const SFString &name, const SFAbstractNode &N)
{
  _nameDict[name].push_back(N);
}

void NameDict::remove(SFAbstractNode N)
{
  SFString name = getNameOfNode(N);
  _nameDict[name].remove(N);
  if (_nameDict[name].empty())
    _nameDict.erase(name);
}

void NameDict::makeDEFNonAmbiguous()
{
  map<SFString, list<SFAbstractNode>, ltSFString> nameDictCopy;
  nameDictCopy.swap(_nameDict);
  
  for (map<SFString, list<SFAbstractNode>, ltSFString>::iterator it = nameDictCopy.begin(); it != nameDictCopy.end(); ++it)
  {
    SFString name = (*it).first;
    SFString nname = name;    
    
    unsigned int k = 1;
    
    for (list<SFAbstractNode>::const_iterator itNode = (*it).second.begin(); itNode != (*it).second.end(); ++itNode)
    {
      while (_nameDict.find(nname) != _nameDict.end())
      {
        nname = name + "_" + SFString::number(k);
        ++k;
      }
      _nameDict[nname].push_back(*itNode);
    }  
  }
}

SFAbstractNode NameDict::getNodeOfName(const SFString &name) const
{
  map<SFString, std::list<SFAbstractNode>, ltSFString>::const_iterator it = _nameDict.find(name);
  if (it != _nameDict.end())
    return (*it).second.back();
  return 0;  
}

SFString NameDict::getNameOfNode(const SFAbstractNode &N) const
{
  for (map<SFString, std::list<SFAbstractNode>, ltSFString>::const_iterator it = _nameDict.begin(); it != _nameDict.end(); ++it)
  {
    for (list<SFAbstractNode>::const_iterator itNode = (*it).second.begin(); itNode != (*it).second.end(); ++itNode)
    {
      if (*itNode == N)
        return (*it).first;
    }  
  }
  
  return SFString("");
}

void NameDict::printDict() const
{
  cout << "DEF dictionary" << endl;
  for (map<SFString, list<SFAbstractNode>, ltSFString>::const_iterator it = _nameDict.begin(); it != _nameDict.end(); ++it)
  {
    for (list<SFAbstractNode>::const_iterator itNode = (*it).second.begin(); itNode != (*it).second.end(); ++itNode)
      cout << (*it).first << " -> " << *itNode << endl;
  }
  cout << "end DEF dictionary" << endl;  
} 

}
