#include "Walker.h"
#include "X3DComponentWalker.h"
#include "WalkingFunction.h"
#include "Visitor.h"
#include "AbstractWalker.h"
#include "Type.h"

using namespace X3DTK;
using namespace std;

Walker::Walker()
: X3DActor(), _visitor(0)
{
  Type::addActor(this);
  setComponentWalker(new AbstractWalker());
}

Walker::~Walker()
{
  if (autoDelete)
    for (list<X3DComponentWalker *>::iterator it = _componentList.begin(); it != _componentList.end(); ++it)
    {
      (*it)->removeOneActor();
      if ((*it)->getActorNumber() == 0)
        delete *it;
    }    
 
  Type::removeActor(this);
}

void Walker::setVisitor(Visitor *visitor)
{
  _visitor = visitor;
  for (list<X3DComponentWalker *>::iterator it = _componentList.begin(); it != _componentList.end(); ++it)
    (*it)->setVisitor(visitor);
}

void Walker::setComponentWalker(X3DComponentWalker *component)
{
  //replace the equivalent component.
  X3DComponentWalker *replacedComponent = 0;
  bool replaced = false;
  for (list<X3DComponentWalker *>::iterator it = _componentList.begin(); it != _componentList.end(); ++it)
  {
    if ((*it)->getComponent()->getEncodedName() == component->getComponent()->getEncodedName())
    {
      replacedComponent = *it;
      *it = (X3DComponentWalker *)component; 
      replacedComponent->removeOneActor();
      component->addOneActor();      
      replaced = true;
    }  
  }
  
  if (replacedComponent == component)
    return;
  
  if (!replaced)
  {
    _componentList.push_back(component);
    component->addOneActor();
  }  
  
  //remove replaced component.
  if ((autoDelete) && (replacedComponent != 0) && (replacedComponent->getActorNumber() == 0))
    delete replacedComponent;
        
  component->setWalker(this);
  component->setVisitor(_visitor);
  
  //iterating the types.
  for (unsigned int id = 0; id < _walkingArray.size(); ++id)
    _walkingArray[id] = getWalkingFunctionOf(Type::getTypeOfId(id));
}

void Walker::reset()
{
  //remove components
  if (autoDelete)
    for (list<X3DComponentWalker *>::iterator it = _componentList.begin(); it != _componentList.end(); ++it)
    {
      (*it)->removeOneActor();
      if ((*it)->getActorNumber() == 0)
        delete *it;
    }    
 
  _componentList.clear();
}

void Walker::addType(const Type *type)
{
  //resize the array.
  if (static_cast<int>(_walkingArray.size()) <= type->getId())
    _walkingArray.resize(type->getId() + 1);

  _walkingArray[type->getId()] = getWalkingFunctionOf(type);
}

WalkingFunction *Walker::getWalkingFunctionOf(const Type *type) const
{
  Type *t = (Type *)type;
  WalkingFunction *WF = 0;
  while (t != 0)
  {
    for (list<X3DComponentWalker *>::const_iterator it = _componentList.begin(); it != _componentList.end(); ++it)
    {
      WF = (*it)->getWalkingFunctionOf(t);
      if (WF != 0)
        break;
    }
    if (WF != 0)
      break;
    
    t = t->getParent();
  }
  
  return WF;
}

Walker *X3DTK::joinWalkers(Walker *N0, Walker *N1)
{
  Walker *N = new Walker();
  
  for (list<X3DComponentWalker *>::const_iterator it = N0->_componentList.begin(); it != N0->_componentList.end(); ++it)
    N->setComponentWalker(*it);
  
  for (list<X3DComponentWalker *>::const_iterator it = N1->_componentList.begin(); it != N1->_componentList.end(); ++it)
    N->setComponentWalker(*it);
  
  return N;  
}
