#include "X3DBaseTypes.h"

#include <stdio.h>
#include <sstream>

using namespace X3DTK;
using namespace std;

class SFStringStream
{
public:
  SFStringStream(const SFString &s);
  float getNextFloat();
  int getNextInt();

  inline bool end() {return isEnd;};
  inline float toFloat(char c) 
  {
    switch(c)
    {
      case '0':
        return 0.0f;
      case '1':
        return 1.0f;
      case '2':
        return 2.0f;
      case '3':
        return 3.0f;
      case '4':
        return 4.0f;
      case '5':
        return 5.0f;          
      case '6':
        return 6.0f;
      case '7':
        return 7.0f;
      case '8':
        return 8.0f;      
      case '9':
        return 9.0f;      
      default:
        return 0.0f;          
    }
  }
  
  inline int toInt(char c) 
  {
    switch(c)
    {
      case '0':
        return 0;
      case '1':
        return 1;
      case '2':
        return 2;
      case '3':
        return 3;
      case '4':
        return 4;
      case '5':
        return 5;          
      case '6':
        return 6;
      case '7':
        return 7;
      case '8':
        return 8;      
      case '9':
        return 9;      
      default:
        return 0;          
    }
  }
  
private:
  const SFString &s;
  string::const_iterator itChar;
  bool isEnd;
};

SFStringStream::SFStringStream(const SFString &ss)
: s(ss), isEnd(false)
{
  itChar = s.begin();
  
  while ((itChar != s.end()) 
      && ((*itChar == ' ') || (*itChar == ',')))
    ++itChar;
  
  isEnd = (itChar == s.end());
}

float SFStringStream::getNextFloat()
{
  bool sign;
  if (*itChar == '-')
  {
    sign = false;
    ++itChar;
  }
  else
  {
    sign = true;
    if (*itChar == '+')
      ++itChar;
  }
  
  float f = 0.0f;
    
  while ((*itChar != ' ') 
      && (*itChar != ',')
      && (*itChar != '.')
      && (*itChar != 'e')
      && (*itChar != 'E')
      && (itChar != s.end()))
  {
    f = 10.0f*f + toFloat(*itChar);
    ++itChar;
  }
  
  float n = 0.1f;
  if (*itChar == '.')
  {
    ++itChar;
    while ((*itChar != ' ') 
        && (*itChar != ',')
        && (*itChar != 'e')
        && (*itChar != 'E')
        && (itChar != s.end()))
    {
      f += n*toFloat(*itChar);  
      n *= 0.1f;
      ++itChar;
    } 
  }
  
  if ((*itChar == 'e') 
   || (*itChar == 'E'))
  {
    ++itChar;
    bool esign;
    if (*itChar == '-')
    {
      esign = false;
      ++itChar;
    }
    else
    {
      esign = true;
      if (*itChar == '+')
        ++itChar;
    }
 
    int exp = 0;     
    while ((*itChar != ' ') 
        && (*itChar != ',')
        && (itChar != s.end()))
    {
      exp = 10*exp + toInt(*itChar);  
      ++itChar;
    }
    
    if (esign)
      f = f*pow(10.0f, exp);
    else
      f = f*pow(10.0f, -exp);            
  }
  
  if (!sign)
    f = -f;

  while (((*itChar == ' ') || (*itChar == ',')) 
      && (itChar != s.end()))
    ++itChar;
  
  isEnd = (itChar == s.end());
  
  return f;
}

int SFStringStream::getNextInt()
{
  bool sign;
  if (*itChar == '-')
  {
    sign = false;
    ++itChar;
  }
  else
  {
    sign = true;
    if (*itChar == '+')
      ++itChar;
  }
  
  int i = 0;
    
  while ((*itChar != ' ') 
      && (*itChar != ',')
      && (*itChar != '.')
      && (*itChar != 'e')
      && (*itChar != 'E')
      && (itChar != s.end()))
  {
    i = 10*i + toInt(*itChar);
    ++itChar;
  }
  
  if ((*itChar == 'e') 
   || (*itChar == 'E'))
  {
    ++itChar;
    if (*itChar == '+')
      ++itChar;
    
    int exp = 0;     
    while ((*itChar != ' ') 
        && (*itChar != ',')
        && (itChar != s.end()))
    {
      exp = 10*exp + toInt(*itChar);  
      ++itChar;
    }
    
    i = i*static_cast<int>(pow(10.0f, exp));
  }
  
  if (!sign)
    i = -i;

  while (((*itChar == ' ') || (*itChar == ',')) 
      && (itChar != s.end()))
    ++itChar;
  
  isEnd = (itChar == s.end());
  
  return i;
}

MFString::MFString(const SFString &s)
{
  string token;
  string::const_iterator itChar = s.begin();
  size_t counter = 0;
  size_t begin = 0;
  size_t size = 0;
  
  while (itChar != s.end())
  {
    if (*itChar == '\"')
    {
      ++itChar;
      ++counter;
      begin = counter;
      size = 0;
      
      while ((*itChar != '\"') && (itChar != s.end()))
      {    
        ++itChar;
        ++counter;
        ++size;
      }
      token = s.substr(begin, size);
      this->push_back(token);
    }
    ++itChar;
    ++counter;
  }

}

MFBool::MFBool(const SFString &)
{
}

MFInt32::MFInt32(size_type n, const SFInt32 &V)
: vector<SFInt32>(n, V)
{
}

MFInt32::MFInt32(const SFString &s)
{
  SFStringStream ss(s);
    
  while (!ss.end())
    push_back(ss.getNextInt());
}

MFFloat::MFFloat(size_type n, const SFFloat &V)
: vector<SFFloat>(n, V)
{
}

MFFloat::MFFloat(const SFString &s)
{
  SFStringStream ss(s);
    
  while (!ss.end())
    push_back(ss.getNextFloat());
    
}

MFDouble::MFDouble(const SFString &)
{
}

SFColorRGBA::SFColorRGBA(SFFloat r, SFFloat g, SFFloat b, SFFloat a)
{
  this->r = r;
  this->g = g;
  this->b = b;
  this->a = a;
}

bool X3DTK::operator== (const SFColorRGBA &r1, const SFColorRGBA &r2)
{
  return ((r1.r == r2.r) && (r1.g == r2.g) && (r1.b == r2.b) && (r1.a == r2.a));
}

bool X3DTK::operator!= (const SFColorRGBA &r1, const SFColorRGBA &r2)
{
  return ((r1.r != r2.r) || (r1.g != r2.g) || (r1.b != r2.b) || (r1.a != r2.a));
}

const SFColorRGBA SFColorRGBA::NULLCOLOR = SFColorRGBA(0.0f, 0.0f, 0.0f, 0.0f);

SFColorRGBA::SFColorRGBA(const SFString &s)
{
  SFStringStream ss(s);
  
  r = ss.getNextFloat();
  g = ss.getNextFloat();
  b = ss.getNextFloat();
  a = ss.getNextFloat();
}

SFString toSFString(const SFColorRGBA c)
{
  return SFString::number(c.r) + SFString(" ") 
       + SFString::number(c.g) + SFString(" ")
       + SFString::number(c.b) + SFString(" ")
       + SFString::number(c.a);
}

MFColorRGBA::MFColorRGBA(size_type n, const SFColorRGBA &C)
: vector<SFColorRGBA>(n, C)
{
}

MFColorRGBA::MFColorRGBA(const SFString &s)
{
  SFStringStream ss(s);
  
  while (!ss.end())
  {
    SFColorRGBA c;
    c.r = ss.getNextFloat();
    c.g = ss.getNextFloat();
    c.b = ss.getNextFloat();
    c.a = ss.getNextFloat();
    
    push_back(c);
  }
}

SFColor::SFColor(SFFloat r, SFFloat g, SFFloat b)
{
  this->r = r;
  this->g = g;
  this->b = b;
};

bool X3DTK::operator== (const SFColor &r1, const SFColor &r2)
{
  return ((r1.r == r2.r) && (r1.g == r2.g) && (r1.b == r2.b));
}

bool X3DTK::operator!= (const SFColor &r1, const SFColor &r2)
{
  return ((r1.r != r2.r) || (r1.g != r2.g) || (r1.b != r2.b));
}

const SFColor SFColor::NULLCOLOR = SFColor(0.0f, 0.0f, 0.0f);

SFColor::SFColor(const SFString &s)
{
  SFStringStream ss(s);
  
  r = ss.getNextFloat();
  g = ss.getNextFloat();
  b = ss.getNextFloat();
}

SFColor::operator const SFColorRGBA() const
{
  return SFColorRGBA(r, g, b, 1.0f);
}

MFColor::MFColor(const SFString &s)
{
  SFStringStream ss(s);
  
  while (!ss.end())
  {
    SFColor c;
    c.r = ss.getNextFloat();
    c.g = ss.getNextFloat();
    c.b = ss.getNextFloat();
    
    push_back(c);
  }
}

MFColor::operator const MFColorRGBA() const
{
  MFColorRGBA color = MFColorRGBA(this->size());
  MFColorRGBA::iterator itColor = color.begin();
  for (const_iterator it = this->begin(); it != this->end(); ++it)
  {
    *itColor = *it;
    ++itColor;
  }  

  return color;
}

SFVec2d::SFVec2d()
{
}

SFVec2d::SFVec2d(SFDouble x, SFDouble y)
{
  this->x = x;
  this->y = y;
}

SFVec2d::SFVec2d(const SFString &)
{
}

MFVec2d::MFVec2d(const SFString &)
{
}

MFVec2f::MFVec2f(size_type n, const SFVec2f &V)
: vector<SFVec2f>(n, V)
{
}

MFVec2f::MFVec2f(const SFString &s)
: vector<SFVec2f>()
{
  SFStringStream ss(s);
  
  while (!ss.end())
  {
    SFVec2f v;
    v.x = ss.getNextFloat();
    v.y = ss.getNextFloat();
    
    push_back(v);
  }
}

SFVec3d::SFVec3d()
{
}


SFVec3d::SFVec3d(SFDouble x, SFDouble y, SFDouble z)
{
  this->x = x;
  this->y = y;
  this->z = z;
}

SFVec3d::SFVec3d(const SFString &)
{
}

MFVec3d::MFVec3d(const SFString &)
{
}

MFVec3f::MFVec3f(size_type n, const SFVec3f &V)
: vector<SFVec3f>(n, V)
{
}

MFVec3f::MFVec3f(const SFString &s)
: vector<SFVec3f>()
{
  SFStringStream ss(s);
  
  while (!ss.end())
  {
    SFVec3f v;
    v.x = ss.getNextFloat();
    v.y = ss.getNextFloat();
    v.z = ss.getNextFloat();
    
    push_back(v);
  }
}

SFRotation::SFRotation(SFFloat x, SFFloat y, SFFloat z, SFFloat angle)
{
  this->x = x;
  this->y = y;
  this->z = z;
  this->angle = angle;
}

SFRotation::SFRotation(const SFString &s)
{
  SFStringStream ss(s);
  
   x = ss.getNextFloat();
   y = ss.getNextFloat();
   z = ss.getNextFloat();
   angle = ss.getNextFloat();
}

bool X3DTK::operator== (const SFRotation &r1, const SFRotation &r2)
{
  return ((r1.x == r2.x) && (r1.y == r2.y) && (r1.z == r2.z) && (r1.angle == r2.angle));
}

bool X3DTK::operator!= (const SFRotation &r1, const SFRotation &r2)
{
  return ((r1.x != r2.x) || (r1.y != r2.y) || (r1.z != r2.z) || (r1.angle != r2.angle));
}

MFRotation::MFRotation(const SFString &s)
{
  SFStringStream ss(s);
  
  while (!ss.end())
  {
    SFRotation r;
    r.x = ss.getNextFloat();
    r.y = ss.getNextFloat();
    r.z = ss.getNextFloat();
    r.angle = ss.getNextFloat();
    
    push_back(r);
  }
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFString &ms)
{
if (!ms.empty())
  {
    for (unsigned int i = 0; i < ms.size() - 1; ++i)
      o << ms[i] << " ";
  
    o << ms.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFBool &mb)
{
  for (unsigned int i = 0; i < mb.size() - 1; ++i)
  {
    if (mb[i])
      o << "TRUE, ";
    else
      o << "FALSE, ";        
  }  
  
  if (!mb.empty())
  {
    if (mb.back())
      o << "TRUE";
    else
      o << "FALSE";        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFInt32 &mi)
{
  if (!mi.empty())
  {
    for (unsigned int i = 0; i < mi.size() - 1; ++i)
      o << mi[i] << " ";
  
    o << mi.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFFloat &mf)
{
  if (!mf.empty())
  {
    for (unsigned int i = 0; i < mf.size() - 1; ++i)
      o << mf[i] << " ";
    
    o << mf.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFDouble &md)
{
  if (!md.empty())
  {
    for (unsigned int i = 0; i < md.size() - 1; ++i)
      o << md[i] << " ";

    o << md.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFColor &c)
{
  return o << c.r << " " << c.g << " " << c.b;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFColor &mc)
{
  if (!mc.empty())
  {
    for (unsigned int i = 0; i < mc.size() - 1; ++i)
      o << mc[i] << ", ";
    
    o << mc.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFColorRGBA &c)
{
  return o << c.r << " " << c.g << " " << c.b << " " << c.a;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFColorRGBA &mc)
{
  if (!mc.empty())
  {
    for (unsigned int i = 0; i < mc.size() - 1; ++i)
      o << mc[i] << ", ";
    
    o << mc.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFVec2d &v)
{
  return o << v.x << " " << v.y ;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFVec2d &mv)
{
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
      o << mv[i] << ", ";
    
    o << mv.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFVec2f &v)
{
  return o << v.x << " " << v.y;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFVec2f &mv)
{
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
      o << mv[i] << ", ";
  
    o << mv.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFVec3d &v)
{
  return o << v.x << " " << v.y << " " << v.z;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFVec3d &mv)
{
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
      o << mv[i] << ", ";
  
    o << mv.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFVec3f &v)
{
  return o << v.x << " " << v.y << " " << v.z;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFVec3f &mv)
{
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
      o << mv[i] << ", ";
  
    o << mv.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFPoint3f &v)
{
  return o << v.x << " " << v.y << " " << v.z;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFRotation &r)
{
  return o << r.x << " " << r.y << " " << r.z << " " << r.angle;
}

std::ostream& X3DTK::operator<<(std::ostream& o, const MFRotation &mr)
{
  if (!mr.empty())
  {
    for (unsigned int i = 0; i < mr.size() - 1; ++i)
      o << mr[i] << ", ";
  
    o << mr.back();        
  }  
  return o;  
}

std::ostream& X3DTK::operator<<(std::ostream& o, const SFMatrix34f &M)
{
  return o << "| " << M._11 << " " << M._12 << " " << M._13 << " " << M._14 << " |" << endl 
           << "| " << M._21 << " " << M._22 << " " << M._23 << " " << M._24 << " |" << endl 
           << "| " << M._31 << " " << M._32 << " " << M._33 << " " << M._34 << " |" << endl;
}
